part of lex.src.combinator;

/// Expects to parse a sequence of [parsers].
///
/// If [failFast] is `true` (default), then the first failure to parse will abort the parse.
ListParser<T> chain<T>(Iterable<Parser> parsers,
    {bool failFast: true, SyntaxErrorSeverity severity}) {
  return new _Chain<T>(
      parsers, failFast != false, severity ?? SyntaxErrorSeverity.error);
}

class _Alt<T> extends Parser<T> {
  final Parser<T> parser;
  final String errorMessage;
  final SyntaxErrorSeverity severity;

  _Alt(this.parser, this.errorMessage, this.severity);

  @override
  ParseResult<T> parse(SpanScanner scanner, [int depth = 1]) {
    var result = parser.parse(scanner, depth + 1);
    return result.successful
        ? result
        : result.addErrors([
            new SyntaxError(severity ?? SyntaxErrorSeverity.error, errorMessage,
                result.span ?? scanner.emptySpan),
          ]);
  }
}

class _Chain<T> extends ListParser<T> {
  final Iterable<Parser<T>> parsers;
  final bool failFast;
  final SyntaxErrorSeverity severity;

  _Chain(this.parsers, this.failFast, this.severity);

  @override
  ParseResult<List<T>> parse(SpanScanner scanner, [int depth = 1]) {
    var errors = <SyntaxError>[];
    var results = <T>[];
    var spans = <FileSpan>[];
    bool successful = true;

    for (var parser in parsers) {
      var result = parser.parse(scanner, depth + 1);

      if (!result.successful) {
        if (parser is _Alt) errors.addAll(result.errors);

        if (failFast) {
          return new ParseResult(this, false, result.errors);
        }

        successful = false;
      }

      results.add(result.value);

      if (result.span != null) spans.add(result.span);
    }

    FileSpan span;

    if (spans.isNotEmpty) {
      span = spans.reduce((a, b) => a.expand(b));
    }

    return new ParseResult<List<T>>(
      this,
      successful,
      errors,
      span: span,
      value: new List<T>.unmodifiable(results),
    );
  }
}
